@@ -133,12 +133,14 @@ def insert_sql(
   columns = ', '.join(c for (c, _) in assign)
   placeholder_list = []
   params = []
+  placeholder = 1
 
-  for (idx, (_, val)) in enumerate(assign):
+  for (_, val) in assign:
     if isinstance(val, Literal):
       placeholder_list.append(val.value)
     else:
-      placeholder_list.append('${}'.format(idx + 1))
+      placeholder_list.append('${}'.format(placeholder))
+      placeholder += 1
       params.append(val)
   placeholders = ', '.join(placeholder_list)
 
