@@ -1,6 +1,6 @@
 '''Contains the DB wrapper class.'''
 from contextlib import (asynccontextmanager)
-from typing import (Any, AsyncIterator, Optional, List, cast)
+from typing import (Any, AsyncIterator, Callable, Optional, List, cast)
 from logging import (Logger, getLogger)
 from asyncpg import (Connection, connect)
 import apgw.builder as builder
@@ -15,6 +15,7 @@ class DB:
       *,
       auto_open: bool = False,
       logger: Logger = getLogger('apgw'),
+      _connect: Callable[..., Connection] = connect,
   ) -> None:
     '''Initializes the DB instance.
 
@@ -22,8 +23,9 @@ class DB:
     :param logger: a Logger instance
     '''
     self.conn_args = conn_args
-    self.auto_open = auto_open
+    self._auto_open = auto_open
     self.logger = logger
+    self._connect = _connect
     self._conn: Optional[Connection] = None
 
   @property
@@ -40,15 +42,22 @@ class DB:
   @property
   def _should_auto_open(self) -> bool:
     '''Returns True if the connection can and should be opened automatically.'''
-    return self.auto_open and not self.is_open
+    return self._auto_open and not self.is_open
+
+  async def auto_open(self) -> bool:
+    '''Opens the database if it should auto open.'''
+    if self._should_auto_open:
+      await self.open()
+      return True
+    return False
 
   async def open(self) -> 'DB':
     '''Opens the connection.'''
     if not self._conn:
       if isinstance(self.conn_args, str):
-        self._conn = await connect(dsn=self.conn_args)
+        self._conn = await self._connect(dsn=self.conn_args)
       else:
-        self._conn = await connect(**self.conn_args)
+        self._conn = await self._connect(**self.conn_args)
     return self
 
   async def close(self) -> 'DB':
@@ -60,6 +69,8 @@ class DB:
 
   @asynccontextmanager
   async def transaction(self) -> AsyncIterator['DB']:
+    '''Run a command in a database transaction.'''
+    await self.auto_open()
     async with self.conn.transaction():
       yield self
 
@@ -89,8 +100,7 @@ class DB:
 
     self._log(sql, params)
 
-    if self._should_auto_open:
-      await self.open()
+    await self.auto_open()
 
     if params:
       return await self.conn.fetch(sql, *params)
@@ -108,8 +118,7 @@ class DB:
 
     self._log(sql, params)
 
-    if self._should_auto_open:
-      await self.open()
+    await self.auto_open()
 
     if params:
       return await self.conn.fetchrow(sql, *params)
